unit IALiveIcon;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,extctrls;

type
  TIALiveIcon = class(TComponent)
  private
    FIntTimer : TTimer;
    FCurrNumber  : Integer;
    FCurrRepeats : Integer;

    IsStop : Boolean;

    FLiveBitmap: TBitmap;
    FCadrNumber: Integer;
    FRepeatNumber: Integer;
    FTransparent: Boolean;
    FTransparentColorX: Integer;
    FTransparentColorY: Integer;
    FFrameTimeDelay: Integer;
    procedure SetLiveBitmap(Value: TBitmap);
    procedure SetCadrNumber(Value: Integer);
    procedure SetRepeatNumber(Value: Integer);
    procedure SetTransparent(Value: Boolean);
    procedure SetTransparentColorX(Value: Integer);
    procedure SetTransparentColorY(Value: Integer);
    procedure SetFrameTimeDelay(Value: Integer);

    procedure OnIntTimer(Sender: TObject);
    procedure GetCurrBMP(var ABitmap : TBitmap);
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Stop;
    procedure Continue;
    procedure StartFromBegin;

    procedure BeginTimer;
    procedure FreeTimer;
  published
    { Published declarations }
    property LiveBitmap : TBitmap read FLiveBitmap write SetLiveBitmap;
    property CadrNumber : Integer read FCadrNumber write SetCadrNumber default 1;
    property RepeatNumber : Integer read FRepeatNumber write SetRepeatNumber default -1;
    property Transparent : Boolean read FTransparent write SetTransparent default false;
    property TransparentColorX : Integer read FTransparentColorX write SetTransparentColorX default 0;
    property TransparentColorY : Integer read FTransparentColorY write SetTransparentColorY default 0;
    property FrameTimeDelay : Integer read FFrameTimeDelay write SetFrameTimeDelay default 100;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('TIA', [TIALiveIcon]);
end;

{ TIALiveIcon }

procedure TIALiveIcon.BeginTimer;
begin
  if FIntTimer=nil then
    begin
      FIntTimer := TTimer.Create(self);
      FIntTimer.OnTimer:=OnIntTimer;
      FIntTimer.Interval:=FFrameTimeDelay;
      FIntTimer.Enabled:=TRUE;
    end;
end;

procedure TIALiveIcon.Continue;
begin
  IsStop:=false;
  BeginTimer;
end;

constructor TIALiveIcon.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLiveBitmap:=TBitmap.Create;
  FLiveBitmap.PixelFormat:=pf24bit;
  FLiveBitmap.Width:=0;
  FLiveBitmap.height:=0;
  FCadrNumber:=1;
  FRepeatNumber:=-1;
  FTransparent:=false;
  FTransparentColorX:=0;
  FTransparentColorY:=0;
  FFrameTimeDelay:=100;
  IsStop:=false;

  FCurrNumber:=0;
  FCurrRepeats:=0;
  OnIntTimer(self);
  FIntTimer := nil;
  BeginTimer;
end;

destructor TIALiveIcon.Destroy;
begin
  FreeTimer;
  FLiveBitmap.free;
  inherited Destroy;
end;

procedure TIALiveIcon.FreeTimer;
begin
  if FIntTimer<>nil then
    begin
      FIntTimer.Enabled:=False;
      FIntTimer.free;
      FIntTimer:=nil;
    end;
end;

procedure TIALiveIcon.GetCurrBMP(var ABitmap: TBitmap);
begin
  ABitmap.Width:=FLiveBitmap.Width div FCadrNumber;
  ABitmap.Height:=FLiveBitmap.Height;
  if (ABitmap.Width>0) and (ABitmap.height>0) then
  ABitmap.Canvas.Draw(-FCurrNumber*ABitmap.Width,0,FLiveBitmap);
end;

procedure TIALiveIcon.OnIntTimer(Sender: TObject);
var
  p1,p2,p3 : pByteArray;
  B1,B2,B3 : TBitmap;
  i,j,w,h : integer;
  Rc1,Gc1,Bc1 : Byte;
  Ico1 : HICON;
  IcoInfo : TICONINFO;
  MyIcon : TIcon;
begin
  //here we draw....
  if  (self.Owner<>nil) and (self.Owner is TForm) and
      ((csDesigning in self.ComponentState) or
       (FRepeatNumber<0) or
       (FCurrRepeats<FRepeatNumber)) and (not IsStop) then
    begin
      //We get curr cadr...
      B3:=TBitmap.Create;
      B3.PixelFormat:=pf24bit;
      B3.width:=0;
      B3.height:=0;
      GetCurrBMP(B3);
      //We define the 2 bitmaps for Icon....
      B1:=TBitmap.Create;
      B1.PixelFormat:=pf24Bit;
      B1.Width:=16;
      B1.Height:=32;
      B2:=TBitmap.Create;
      B2.PixelFormat:=pf24Bit;
      B2.Width:=16;
      B2.Height:=16;


      //we draw this bitmaps...
      if FTRansparent then
        begin
          B1.Canvas.brush.Color:=clWhite;
          B1.Canvas.FillRect(rect(0,0,16,16));
          B2.Canvas.brush.Color:=clBlack;
          B2.Canvas.FillRect(rect(0,0,16,16));
          if (FLiveBitmap.Width>0) and (FLiveBitmap.height>0) then
            begin
              p1:=FLiveBitmap.ScanLine[FTransparentColorY];
              Rc1:=p1[FTransparentColorX*3];
              Gc1:=p1[FTransparentColorX*3+1];
              Bc1:=p1[FTransparentColorX*3+2];
              if B3.Width<16 then w:=B3.width else w:=16;
              if B3.height<16 then h:=B3.height else h:=16;
              for i:=0 to h-1 do
                begin
                  p1:=B1.ScanLine[i];
                  p2:=B2.ScanLine[i];
                  p3:=B3.ScanLine[i];
                  for j:=0 to w-1 do
                    begin
                      if not((p3[j*3]=Rc1) and (p3[j*3+1]=Gc1) and (p3[j*3+2]=Bc1)) then
                        begin
                          p1[j*3  ]:=0;
                          p1[j*3+1]:=0;
                          p1[j*3+2]:=0;
                          p2[j*3  ]:=p3[j*3  ];
                          p2[j*3+1]:=p3[j*3+1];
                          p2[j*3+2]:=p3[j*3+2];
                        end;
                    end;
                end;
            end;
        end else
        begin

          B1.Canvas.brush.Style:=bssolid;
          B1.Canvas.brush.Color:=clWhite;
          B1.Canvas.FillRect(rect(0,0,16,31));
          B1.Canvas.brush.Color:=clblack;
          B1.Canvas.pen.Color:=clblack;
          if (FLiveBitmap.Width>0) and (FLiveBitmap.height>0) then
          B1.Canvas.Rectangle(0,0,B3.width,B3.height);
          B2.Canvas.brush.Color:=clblack;
          B2.Canvas.FillRect(rect(0,0,16,16));
          if (FLiveBitmap.Width>0) and (FLiveBitmap.height>0) then
          B2.Canvas.Draw(0,0,B3);
        end;
      //Now we will create new icon and show it...
      MyIcon:=TIcon.Create;
      ICOInfo.fIcon:=true;
      IcoInfo.xHotspot:=8;
      IcoInfo.yHotspot:=8;
      ICoInfo.hbmMask :=B1.MaskHandle;
      ICoInfo.hbmColor:=B2.Handle;
      ICo1:=CreateIConIndirect(ICoInfo);
      MyIcon.ReleaseHandle;
      MyIcon.Handle:=ICo1;
      (self.Owner as TForm).icon.Assign(MyICon);


      B1.free;
      B2.free;
      B3.free;
      MyIcon.Free;
      //here we count next cadr data
      inc(FCurrNumber);
      if FCurrNumber>=FCadrNumber then
        begin
          FCurrNumber:=0;
          if FRepeatNumber>0 then
            begin
              inc(FCurrRepeats);
            end;
        end;
    end else
    begin
      FreeTimer;
    end;
end;

procedure TIALiveIcon.SetCadrNumber(Value: Integer);
begin
  if (Value<>FCadrNumber) and (Value>=1) and (Value<=FLiveBitmap.Width) then
    begin
      FCadrNumber := Value;
      if (csDesigning in self.ComponentState) then StartFromBegin;
    end;
end;

procedure TIALiveIcon.SetFrameTimeDelay(Value: Integer);
begin
  if (FFrameTimeDelay<>Value) and (Value>=20) then
    begin
      FFrameTimeDelay := Value;
      FIntTimer.Interval:=FFrameTimeDelay;
      if (csDesigning in self.ComponentState) then StartFromBegin;
    end;
end;

procedure TIALiveIcon.SetLiveBitmap(Value: TBitmap);
begin
  if (Value<>nil) and assigned(Value) and (Value.width>0) and (Value.height>0) then
    begin
      FLiveBitmap.Width:=Value.Width;
      FLiveBitmap.Height:=Value.Height;
      if (Value.width>0) and (Value.height>0) then
      FLiveBitmap.Canvas.Draw(0,0,Value);
    end else
    begin
      FLiveBitmap.Width:=0;
      FLiveBitmap.Height:=0;
    end;
  if not((FCadrNumber>=1) and (FCadrNumber<=FLiveBitmap.Width)) then
    begin
      FCadrNumber := 1;
    end;
  if not((FTransparentColorX>=0) and (FTransparentColorX<FLiveBitmap.Width)) then
    begin
      FTransparentColorX := 0;
    end;
  if not((FTransparentColorY>=0) and (FTransparentColorY<FLiveBitmap.Height)) then
    begin
      FTransparentColorY := 0;
    end;
  if (csDesigning in self.ComponentState) then StartFromBegin;
end;

procedure TIALiveIcon.SetRepeatNumber( Value: Integer);
begin
  if (Value<>FRepeatNumber) and (FRepeatNumber>=-1) then
    begin
      FRepeatNumber := Value;
      if (csDesigning in self.ComponentState) then StartFromBegin;
    end;
end;

procedure TIALiveIcon.SetTransparent( Value: Boolean);
begin
  if (Value<>FTransparent) then
    begin
      FTransparent := Value;
      if (csDesigning in self.ComponentState) then StartFromBegin;
    end;
end;

procedure TIALiveIcon.SetTransparentColorX( Value: Integer);
begin
  if (Value<>FTransparentColorX) and (Value>=0) and (Value<FLiveBitmap.Width) then
    begin
      FTransparentColorX := Value;
      if (csDesigning in self.ComponentState) then StartFromBegin;
    end;
end;

procedure TIALiveIcon.SetTransparentColorY( Value: Integer);
begin
  if (Value<>FTransparentColorY) and (Value>=0) and (Value<FLiveBitmap.height) then
    begin
      FTransparentColorY := Value;
      if (csDesigning in self.ComponentState) then StartFromBegin;
    end;
end;

procedure TIALiveIcon.StartFromBegin;
begin
  IsStop:=false;
  FCurrNumber:=0;
  FCurrRepeats:=0;
  BeginTimer;
end;

procedure TIALiveIcon.Stop;
begin
  IsStop:=True;
  FreeTimer;
end;

end.
